use std::env;
use leptos::*;
use serde::{Deserialize, Serialize};

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct MarkdownPage {
    pub title: String,
    pub slug: String,
    pub content: String,
}

#[derive(Clone, Debug, Serialize, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct MarkdownPageLanguagesWrapper {
    pub internal_title: String,
    pub slug: String,
    pub english: MarkdownPage,
}

#[server(GetRichTextPage, "/api", "GetJson")]
pub async fn get_markdown_page(slug: String) -> Result<MarkdownPageLanguagesWrapper, ServerFnError> {
    let contentful_client = get_contentful_client();
    let builder = contentful::QueryBuilder::new()
        .content_type_is("markdownPageLanguagesWrapper")
        .include(10)
        .field_equals("fields.slug", &slug);
    // let query_string = format!("?content_type=markdownPageLanguagesWrapper&include=10&fields.slug={}", slug);
    // let pages = contentful_client.get_entries_by_query_string::<RichTextPageLanguagesWrapper>(&query_string).await;
    let pages = contentful_client.get_entries::<MarkdownPageLanguagesWrapper>(Some(builder)).await;
    match pages {
        Ok(found_pages) => {
            if found_pages.len() != 1 {
                return Err(ServerFnError::ServerError("Found none or more than one".to_string()));
            }
            let english_content = found_pages[0].english.content.clone();
            dbg!(english_content);
            Ok(found_pages[0].clone())
        },
        Err(e) => Err(ServerFnError::ServerError(e.to_string())),
    }
}

pub fn get_contentful_client() -> contentful::ContentfulClient {
    let access_token = env::var("CONTENTFUL_ACCESS_TOKEN").unwrap();
    const SPACE_ID: &str = "e3magj9g6dp1";
    // let base_url = "https://cdn.contentful.com/spaces";
    // const ENVIRONMENT_ID: &str = "master";
    // return Client::new(&access_token, SPACE_ID, base_url, ENVIRONMENT_ID);
    return contentful::ContentfulClient::new(&access_token, SPACE_ID);
}
